pub(crate) fn binary_array_value(
array: &dyn Array,
index: usize,
) -> Option<&[u8]>Expand description
Returns the logical bytes at the given index from a binary-like array, resolving dictionary
and run-end encodings. Returns None for nulls or if the logical values aren’t binary-like.