variant_from_arrays_at

Function variant_from_arrays_at 

pub(crate) fn variant_from_arrays_at<'m, 'v>(
    metadata: &'m dyn Array,
    value: &'v dyn Array,
    index: usize,
) -> Option<Variant<'m, 'v>>
Returns a [Variant] from a metadata and value byte arrays, returns None if one of them is of invalid type.