pub(crate) struct StructArrayBuilder {
fields: Vec<FieldRef>,
arrays: Vec<ArrayRef>,
nulls: Option<NullBuffer>,
}Expand description
Builds struct arrays from component fields
TODO: move to arrow crate
Fields
fields: Vec<FieldRef>arrays: Vec<ArrayRef>nulls: Option<NullBuffer>Implementations
Sourceimpl StructArrayBuilder
impl StructArrayBuilder
pub fn new() -> Self
Sourcepub fn with_field(
self,
field_name: &str,
array: ArrayRef,
nullable: bool,
) -> Self
pub fn with_field( self, field_name: &str, array: ArrayRef, nullable: bool, ) -> Self
Add an array to this struct array as a field with the specified name.
Sourcepub fn with_field_ref(self, field: FieldRef, array: ArrayRef) -> Self
pub fn with_field_ref(self, field: FieldRef, array: ArrayRef) -> Self
Add an array to this struct array using a caller-supplied [FieldRef].
Use this when the field carries metadata (e.g. an extension type) that would be lost if the field were synthesized from the array’s data type alone.
Sourcepub fn with_nulls(self, nulls: NullBuffer) -> Self
pub fn with_nulls(self, nulls: NullBuffer) -> Self
Set the null buffer for this struct array.
pub fn build(self) -> StructArray