pub struct VariantArrayBuilder {
nulls: NullBufferBuilder,
metadata_builder: WritableMetadataBuilder,
metadata_offsets: Vec<usize>,
value_builder: ValueBuilder,
value_offsets: Vec<usize>,
fields: Fields,
}Expand description
A builder for VariantArray
This builder is used to construct a VariantArray and allows APIs for
adding metadata
This builder always creates a VariantArray using [BinaryViewArray] for both
the metadata and value fields.
TODO
- Support shredding: https://github.com/apache/arrow-rs/issues/7895
Example:
// Create a new VariantArrayBuilder with a capacity of 100 rows
let mut builder = VariantArrayBuilder::new(100);
// append variant values
builder.append_variant(Variant::from(42));
// append a null row (note not a Variant::Null)
builder.append_null();
// append an object to the builder using VariantBuilderExt methods directly
builder.new_object()
.with_field("foo", "bar")
.finish();
// bulk insert a list of values
// `Option::None` is a null value
builder.extend([None, Some(Variant::from("norm"))]);
// create the final VariantArray
let variant_array = builder.build();
assert_eq!(variant_array.len(), 5);
// // Access the values
// row 1 is not null and is an integer
assert!(!variant_array.is_null(0));
assert_eq!(variant_array.value(0), Variant::from(42i32));
// row 1 is null
assert!(variant_array.is_null(1));
// row 2 is not null and is an object
assert!(!variant_array.is_null(2));
let value = variant_array.value(2);
let obj = value.as_object().expect("expected object");
assert_eq!(obj.get("foo"), Some(Variant::from("bar")));
// row 3 is null
assert!(variant_array.is_null(3));
// row 4 is not null and is a short string
assert!(!variant_array.is_null(4));
let value = variant_array.value(4);
assert_eq!(value, Variant::ShortString(ShortString::try_new("norm").unwrap()));Fields
nulls: NullBufferBuilderNulls
metadata_builder: WritableMetadataBuilderbuilder for all the metadata
metadata_offsets: Vec<usize>ending offset for each serialized metadata dictionary in the buffer
value_builder: ValueBuilderbuilder for values
value_offsets: Vec<usize>ending offset for each serialized variant value in the buffer
fields: FieldsThe fields of the final StructArray
TODO: 1) Add extension type metadata TODO: 2) Add support for shredding
Implementations
Sourceimpl VariantArrayBuilder
impl VariantArrayBuilder
pub fn new(row_capacity: usize) -> Self
Sourcepub fn build(self) -> VariantArray
pub fn build(self) -> VariantArray
Build the final builder
Sourcepub fn append_null(&mut self)
pub fn append_null(&mut self)
Appends a null row to the builder.
Sourcepub fn append_variant(&mut self, variant: Variant<'_, '_>)
pub fn append_variant(&mut self, variant: Variant<'_, '_>)
Append the [Variant] to the builder as the next row
Sourcefn parent_state(&mut self) -> ParentState<'_, ArrayBuilderState<'_>>
fn parent_state(&mut self) -> ParentState<'_, ArrayBuilderState<'_>>
Creates a builder-specific parent state