pub(crate) enum ShreddedPathStep {
Success(ShreddingState),
Missing,
NotShredded,
}Variants
Success(ShreddingState)
Path step succeeded, return the new shredding state
Missing
The path element is not present in the typed_value column and there is no value column,
so we know it does not exist. It, and all paths under it, are all-NULL.
NotShredded
The path element is not present in the typed_value column and must be retrieved from the value
column instead. The caller should be prepared to handle any value, including the requested
type, an arbitrary “wrong” type, or Variant::Null.