pub(crate) fn all_null_value_column(len: usize) -> ArrayRefExpand description
Returns an all-null binary value column of the given length.
The shredding spec requires the value column to always be present in the
schema, so producers that have no unshredded values to store must still
emit an all-null column. See https://github.com/apache/arrow-rs/issues/10306.