pub(crate) struct MapVariantToArrowRowBuilder<'a> {
entries_field: &'a FieldRef,
key_field: &'a FieldRef,
value_field: &'a FieldRef,
ordered: bool,
key_builder: Box<VariantToArrowRowBuilder<'a>>,
value_builder: Box<VariantToArrowRowBuilder<'a>>,
offsets: Vec<i32>,
current_offset: i32,
nulls: NullBufferBuilder,
cast_options: &'a CastOptions<'a>,
}Expand description
Builder for converting variant objects into Arrow MapArrays.
Each variant object field becomes one map entry: the field name is the map key and the field value is converted to the requested value type. Variant objects store their fields in lexicographic key order, so maps with string keys come out sorted by key.
Fields
entries_field: &'a FieldRefkey_field: &'a FieldRefvalue_field: &'a FieldRefordered: boolkey_builder: Box<VariantToArrowRowBuilder<'a>>value_builder: Box<VariantToArrowRowBuilder<'a>>offsets: Vec<i32>current_offset: i32nulls: NullBufferBuildercast_options: &'a CastOptions<'a>Implementations
Sourceimpl<'a> MapVariantToArrowRowBuilder<'a>
impl<'a> MapVariantToArrowRowBuilder<'a>